@@ -367,77 +367,47 @@ pub mod dp {
     /// Finds the integers from two vectors that sum to the same value.
     /// This method assumes that the two vectors have Many-to-Many relationships.
     /// Each integer of the `keys` vector corresponds to the multiple integers of the `targets` vector.
-    /// With this method, we can find some combinations of the integers.
-    /// `n_shuffle` is the number of trials. For each case, `kays` are shuffled. 
+    /// With this method, we can find combinations of the integers.
     /// # Arguments
     /// * `keys` - An array.
     /// * `targets` - An array.
-    /// * `n_shuffle` - The number of trials.
     /// # Example
     ///
     /// ```rust
     ///
     ///use dpss::dp::sequence_matcher_m2m;
-    ///let answer = sequence_matcher_m2m(&mut vec![1980, 2980, 3500, 4000, 1050], &mut vec![1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20], 10);
+    ///let answer = sequence_matcher_m2m(&mut vec![1980, 2980, 3500, 4000, 1050], &mut vec![1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20]);
     ///assert_eq!(answer[0], vec![
     ///    (vec![1050],
     ///     vec![1050]),
     ///
-    ///     (vec![1980],
-    ///     vec![30, 1950]),
-    ///
-    ///     (vec![2980],
-    ///     vec![80, 2900]),
-    /// 
-    ///     (vec![3500],
-    ///     vec![200, 3300]),
-    /// 
-    ///     (vec![4000],
-    ///     vec![20, 3980]),
-    ///
+    ///     (vec![1980, 2980, 3500, 4000],
+    ///     vec![20, 30, 80, 200, 1950, 2900, 3300, 3980]),
     ///
     ///    ]);
     ///assert_eq!(answer[1], vec![
     ///    (vec![1050, 1980, 2980],
     ///     vec![80, 1950, 3980]),
     ///
-    ///     (vec![3500],
-    ///     vec![200, 3300]),
-    ///
-    ///     (vec![4000],
-    ///     vec![20, 30, 1050, 2900]),
+    ///     (vec![3500, 4000],
+    ///     vec![20, 30, 200, 1050, 2900, 3300]),
     ///
     ///    ]);
     /// ```
     pub fn sequence_matcher_m2m(
         keys: &mut Vec<i32>,
-        targets: &mut Vec<i32>,
-        n_candidates: usize,
+        targets: &mut Vec<i32>
     ) -> Vec<Vec<(Vec<i32>, Vec<i32>)>> {
-        use rand::seq::SliceRandom;
 
         let mut group: Vec<(Vec<i32>, Vec<i32>)> = Vec::with_capacity(targets.len());
-        let mut answer: Vec<Vec<(Vec<i32>, Vec<i32>)>> = Vec::with_capacity(n_candidates);
-        let mut rng: rand::rngs::StdRng = rand::SeedableRng::from_seed([13; 32]);
+        let mut answer: Vec<Vec<(Vec<i32>, Vec<i32>)>> = vec![];
         if keys.iter().sum::<i32>() != targets.iter().sum() {
             println!("The sum of the keys must be equal to the sum of the targets.");
             return answer;
         }
         let mut hashmap_fs: HashMap<(Vec<i32>, i32), Vec<Vec<i32>>> = HashMap::new();
-        for _i in 0..n_candidates {
-            let mut key_target_group: HashMap<(Vec<i32>, Vec<i32>), Vec<(Vec<i32>, Vec<i32>)>> = HashMap::new();
-            sequence_matcher_core_m2m(keys, targets, &mut group, &mut answer, 1, keys.len(), targets.len(), &mut hashmap_fs, &mut key_target_group);
-            keys.shuffle(&mut rng);
-        }
-        for i in 0..answer.len(){
-            for j in 0..answer[i].len(){
-                answer[i][j].0.sort();
-                answer[i][j].1.sort();
-            }
-            answer[i].sort();
-            answer[i].dedup();
-
-        }
+        let mut key_target_group: HashMap<(Vec<i32>, Vec<i32>), Vec<(Vec<i32>, Vec<i32>)>> = HashMap::new();
+        sequence_matcher_core_m2m(keys, targets, &mut group, &mut answer, 1, keys.len(), targets.len(), &mut hashmap_fs, &mut key_target_group);
         answer.sort();
         answer.dedup();
         if answer.len() == 0 {
@@ -457,6 +427,8 @@ pub mod dp {
         hashmap_fs: &mut HashMap<(Vec<i32>, i32), Vec<Vec<i32>>>,
         key_target_group: &mut HashMap<(Vec<i32>, Vec<i32>), Vec<(Vec<i32>, Vec<i32>)>>,
     ) {
+        use itertools::Itertools;
+
         if keys.iter().sum::<i32>() != targets.iter().sum() {
             return;
         }
@@ -473,73 +445,86 @@ pub mod dp {
             return;
         }
 
-        let mut sum_key = 0;
-        let mut vec_key = vec![];
-        for i in 0..n_key {
-            sum_key += keys[i];
-            vec_key.push(keys[i].clone())
-        }
-        if targets.iter().max().unwrap() == &0{
-            return;
-        }
-        targets.sort();
-        let set_ = hashmap_fs.entry((targets.clone(), sum_key.clone())).or_insert(find_subset(&targets, sum_key, max_target_length)).clone();
-        if set_.len() == 0 {
-            sequence_matcher_core_m2m(keys, targets, group, answer, n_key + 1, keys.len(), targets.len(), hashmap_fs, key_target_group);
-        }
-        for set in set_ {
-            let mut _set = Vec::from(set.clone());
-            _set.sort();
-            let mut _vec_key = vec_key.clone();
-            _vec_key.sort();
-            group.push((_vec_key, _set));
-            for el in set.clone() {
-                if targets.contains(&el) == false {
+        let key_candidates = (0..keys.len()).powerset().collect::<Vec<_>>();
+        let goldkey = keys.clone();
+        let goldtargets = targets.clone();
+        let goldgroup = group.clone();
+        key_candidates.iter().for_each(|i| {
+            *keys = goldkey.clone();
+            *targets = goldtargets.clone();
+            *group = goldgroup.clone();
+            let mut sum_key = 0;
+            let mut vec_key = vec![];
+            for j in i.iter() {
+                if j >= &keys.len(){
                     return;
                 }
-                vec_remove(targets, el);
+                sum_key += keys[*j];
+                vec_key.push(keys[*j].clone());
             }
-            for i in vec_key.clone() {
-                if keys.contains(&i) == false {
-                    return;
-                }
-                vec_remove(keys, i);
+            if targets.iter().max().unwrap() == &0{
+                return;
             }
-            if keys.len() > 0 && targets.len() > 0 {
-                if key_target_group.contains_key(&(keys.clone(), targets.clone())) {
-                    group.push((keys.clone(), targets.clone()));
-                    for d in key_target_group.get(&(keys.clone(), targets.clone())).unwrap(){
-                        if group.contains(&d) == false {
-                            group.push(d.clone());
-                        }
-                        for i in &d.0 {
-                            if keys.contains(&i) == false {
-                                return;
-                            }
-                            vec_remove(keys, *i);
-                        }
-                        for el in &d.1 {
-                            if targets.contains(&el) == false {
-                                return;
-                            }
-                            vec_remove(targets, *el);
-                        }
+            targets.sort();
+            let set_ = hashmap_fs.entry((targets.clone(), sum_key.clone())).or_insert(find_subset(&targets, sum_key, max_target_length)).clone();
+            if set_.len() == 0 {
+                return;
+            }
+            for set in set_ {
+                let mut _set = Vec::from(set.clone());
+                _set.sort();
+                let mut _vec_key = vec_key.clone();
+                _vec_key.sort();
+                group.push((_vec_key, _set));
+                for el in set.clone() {
+                    if targets.contains(&el) == false {
+                        return;
                     }
-                    answer.push(group.clone());
-                    return;
-                } else {
-                    key_target_group.insert((keys.clone(), targets.clone()), group.clone());
+                    vec_remove(targets, el);
+                }
+                for i in vec_key.clone() {
+                    if keys.contains(&i) == false {
+                        return;
+                    }
+                    vec_remove(keys, i);
+                }
+                if keys.len() > 0 && targets.len() > 0 {
+                    if key_target_group.contains_key(&(keys.clone(), targets.clone())) {
+                        group.push((keys.clone(), targets.clone()));
+                        for d in key_target_group.get(&(keys.clone(), targets.clone())).unwrap(){
+                            if group.contains(&d) == false {
+                                group.push(d.clone());
+                            }
+                            for i in &d.0 {
+                                if keys.contains(&i) == false {
+                                    return;
+                                }
+                                vec_remove(keys, *i);
+                            }
+                            for el in &d.1 {
+                                if targets.contains(&el) == false {
+                                    return;
+                                }
+                                vec_remove(targets, *el);
+                            }
+                        }
+                        answer.push(group.clone());
+                        return;
+                    } else {
+                        key_target_group.insert((keys.clone(), targets.clone()), group.clone());
+                    }
+                }
+                sequence_matcher_core_m2m(keys, targets, group, answer, n_key, keys.len(), targets.len(), hashmap_fs, key_target_group);
+                group.pop();
+                for el in set.clone() {
+                    targets.push(el);
+                }
+                for i in vec_key.clone() {
+                    keys.push(i);
                 }
             }
-            sequence_matcher_core_m2m(keys, targets, group, answer, n_key, keys.len(), targets.len(), hashmap_fs, key_target_group);
-            group.pop();
-            for el in set.clone() {
-                targets.push(el);
-            }
-            for i in vec_key.clone() {
-                keys.push(i);
-            }
-        }
+            return;
+        });
     }
 
     #[test]
@@ -572,48 +557,43 @@ pub mod dp {
         let answer = sequence_matcher_m2m(
             &mut vec![6, 7, 3, 2, -9, -3, 8, 3, 6, -10],
             &mut vec![3, 2, -6, -8, 2, -9, 0, -5, -3, 37],
-            10
         );
         assert_eq!(
             answer[0],
             vec![
-                (vec![-10, -9, -3, 3, 6, 7, 8], vec![2]),
-                (vec![2], vec![2]),
-                (vec![3], vec![3]),
-                (vec![6], vec![-9, -8, -6, -5, -3, 0, 37]),
+                (vec![], vec![-5, 2, 3]),
+                (vec![], vec![0]),
+                (vec![-10, -9, -3, 2, 3, 3, 6, 6, 7, 8], vec![-9, -8, -6, -3, 2, 37]),
             ]
         );
 
         let answer = sequence_matcher_m2m(
             &mut vec![6, 7, 3, 2, -9],
             &mut vec![-3, 8, 3, 6, -5],
-            10
         );
         assert_eq!(
             answer[0],
             vec![
-                (vec![-9, 2, 3, 7], vec![-5, -3, 3, 8]),
-                (vec![6], vec![6]),
+                (vec![], vec![-5, -3, 8]),
+                (vec![-9, 2, 3, 6, 7], vec![3, 6]),
             ]
         );
 
         let answer = sequence_matcher_m2m(
             &mut vec![9, 0, 1, 7, 1],
             &mut vec![7, 2, 8, 0, 1],
-            10
         );
         assert_eq!(
             answer[0],
             vec![
-                (vec![0], vec![0]),
-                (vec![1], vec![1]),
-                (vec![1, 9], vec![2, 8]),
-                (vec![7], vec![7]),
+                (vec![], vec![0]),
+                (vec![0, 1], vec![1]),
+                (vec![1, 7, 9], vec![2, 7, 8]),
             ]
         );
 
         let answer =
-            sequence_matcher_m2m(&mut vec![1, 2, 3, 4, 5], &mut vec![11, -8, 14, -7, 5], 10);
+            sequence_matcher_m2m(&mut vec![1, 2, 3, 4, 5], &mut vec![11, -8, 14, -7, 5]);
         assert_eq!(
             answer[0],
             vec![(vec![1], vec![-8, -7, 5, 11]), (vec![2, 3, 4, 5], vec![14]),]
@@ -622,38 +602,33 @@ pub mod dp {
         let answer = sequence_matcher_m2m(
             &mut vec![1000, 1100, 150, 123, 5, 10],
             &mut vec![2100, 273, 4, 11],
-            10
         );
         assert_eq!(
             answer[0],
             vec![
                 (vec![5, 10], vec![4, 11]),
-                (vec![123, 150], vec![273]),
-                (vec![1000, 1100], vec![2100]),
+                (vec![123, 150, 1000, 1100], vec![273, 2100]),
             ]
         );
         assert_eq!(
             answer[1],
             vec![
-                (vec![5, 10], vec![4, 11]),
-                (vec![123, 150, 1000, 1100], vec![273, 2100]),
+                (vec![5, 10, 123, 150, 1000, 1100], vec![4, 11, 273, 2100]),
             ]
         );
 
         let answer = sequence_matcher_m2m(
             &mut vec![1000, 1100, 150, 123, 5, 10],
             &mut vec![1000, 1200],
-            10
         );
         assert_eq!(answer.len(), 0);
 
-        let answer = sequence_matcher_m2m(&mut vec![-950, 10000], &mut vec![5000, 4000, 50], 10);
+        let answer = sequence_matcher_m2m(&mut vec![-950, 10000], &mut vec![5000, 4000, 50]);
         assert_eq!(answer[0], vec![(vec![-950, 10000], vec![50, 4000, 5000]),]);
 
         let answer = sequence_matcher_m2m(
             &mut vec![99, 68, -74, 72, -38, 22],
             &mut vec![36, -23, -92, 88, 67, 73],
-            10
         );
         assert_eq!(
             answer[0],
@@ -663,7 +638,7 @@ pub mod dp {
             ]
         );
 
-        let answer = sequence_matcher_m2m(&mut vec![1, 2, 3, 4], &mut vec![1, 5], 10);
+        let answer = sequence_matcher_m2m(&mut vec![1, 2, 3, 4], &mut vec![1, 5]);
 
         assert_eq!(answer.len(), 0);
     }
